; $Id: dcstoolsdetectors.pro,v 1.10 2002/10/16 15:40:33 john Exp $
; Written by J.R.D. Copley.
;************************************************************************************************
pro DCSToolsDetectors_handler,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue=state
help=state.help
done=state.done
dpt=state.dpt
contents=state.contents
nhedet=state.nhedet
ntotal=state.ntotal
;
if (event.id eq help) then begin
	res=dialog_message([$
		'Number is the detector ID which runs from 0 to 912.',$
		'Crate, Board, Input, Address, Cable, Rack and Position should be of no interest to the user:',$
		'   Crate, Board and Input specify where the detector is plugged into the data acquisition rack,',$
		'   Address specifies the address bits within the word created when an event occurs in the detector,',$
		'   Cable is the number of the signal cable for the detector,',$
		'   Rack, Bank and Position label the physical location of the detector.',$
		'Bank = -1, 0 or 1 depending whether the detector is in the lower, middle or upper bank.',$
		'"2Theta" is the scattering angle (2theta), >0 (<0) if scattering is to the left (right) of the incident beam.',$
		'Polar is the polar angle to the center of the detector.',$
		'Azim. is the azimuthal angle to the center of the detector.',$
		'','Click on a column label to sort by that column.'],/information)
	return
endif
;
if (event.id eq dpt) then begin
	if (event.type eq 4) then begin
		column=event.sel_left
		if ((column ge 0) and (column lt 12)) then begin
			inarr=contents[*,0:nhedet-1]
			dcs_sort_by,float(inarr),outarr,col=column+1,row=0,error
			contents[*,0:nhedet-1]=string(outarr)
			if (column ne 0 and column ne 4 and column ne 5 and column lt 9) $
				then begin
				; perform secondary sort by angle [column 9] for degenerate $
				; values in the primary sorting column (skip for columns 0,4,5,9,10,11)
				row1=0
				repeat begin
					value=contents[column,row1]
					nrows=n_elements(where(value eq contents[column,*]))
					rown=row1+nrows-1
					inarr=contents[*,row1:rown]
					dcs_sort_by,float(inarr),outarr,col=10,row=0,error
					contents[*,row1:rown]=outarr
					row1=rown+1
				endrep until (row1 ge nhedet)
			endif
			contents[0:8,*]=fix(contents[0:8,*])
			contents[9,*]=string(contents[9,*],format='(f7.2)')
			contents[10,*]=string(contents[10,*],format='(f7.2)')
			contents[11,*]=string(contents[11,*],format='(f7.2)')
			contents[5:9,nhedet:ntotal-1]=""
			state.contents=contents
			widget_control,dpt,set_value=contents
			widget_control,event.top,set_uvalue=state
		endif
	endif
endif
;
if (event.id eq done) then begin
	widget_control,event.top,/destroy
	return
endif
;
end


;************************************************************************************************
pro DCSToolsDetectors,event
;************************************************************************************************
;
compile_opt strictarr
;
widget_control,event.top,get_uvalue=statePtr
davePtr=(*statePtr).davePtr
;
dcs_sysvars,davePtr=davePtr
;
tvlct,r,g,b,/get
rgbptr=ptr_new({r:r,g:g,b:b})
;
nhedet=913
;
inputfile=file_which("dcs_DetPos2.dat",/include_current_dir)
openr,unit,inputfile,/get_lun
;
content=strarr(nhedet+1)
readf,unit,content
close,unit
free_lun,unit
;
content[0]=content[0]+' 0.0'
dcs_getangles,nhedet=nhedet,angles,banks,azim_angles
content[1:nhedet]=content[1:nhedet]+$
	strcompress(string(abs(angles),format='(f7.2)'))+$
	strcompress(string(azim_angles,format='(f7.2)'))
;
contents=strarr(12,nhedet)
ntotal=nhedet
column_labels=['Number','Crate','Board','Input','Address',$
	'Cable','Rack','Bank','Position','"2Theta"','Polar','Azim.']
;
for i=0,nhedet-1 do begin
	temp=strsplit(content[i+1]," ",/extract)
	contents[0,i]=temp[0]
	if (fix(temp[1]) lt 14) then contents[1,i]=0 else contents[1,i]=1
	contents[2:11,i]=temp[1:10]
endfor
;
;;; Optional lines.
nhigh=930-nhedet
contentsh=strarr(12,nhigh)
contentsh[0,*]=indgen(nhigh)+nhedet
contentsh[1,*]=1
contentsh[2,*]=indgen(nhigh)+14
contentsh[3,*]=29
contentsh[4,*]=indgen(nhigh)+942
contents=transpose([transpose(contents),transpose(contentsh)])
ntotal=ntotal+nhigh
for i=nhedet,ntotal-1 do contents[*,i]=strcompress(contents[*,i],/remove_all)
;;;
;
;dcs_sysdep,fonts=fonts
;
row_labels=string(indgen(ntotal)+1)
alignment=bytarr(13,ntotal)+1
alignment[9:11,*]=2
tlb=widget_base(title='Detector Grouping',/col,group_leader=event.top,/modal)
	help=widget_button(tlb,value="HELP");,font=fonts[0])
	dpt=widget_table(tlb,xsize=13,ysize=ntotal,x_scroll_size=13,y_scroll_size=26,$
		column_widths=[50,50,50,50,50,50,50,50,50,50,50,50,5],column_labels=column_labels,$
		alignment=alignment,row_labels=row_labels,value=contents,/sensitive,/all_events)
	done=widget_button(tlb,value="DONE");,font=fonts[0])
centertlb,tlb
widget_control,tlb,/realize
;
state={help:help,done:done,dpt:dpt,contents:contents,nhedet:nhedet,ntotal:ntotal}
widget_control,tlb,set_uvalue=state
;
xmanager,"DCSToolsDetectors",tlb,event_handler="DCSToolsDetectors_handler"
;
dave_makeDavePtr,davePtr=davePtr
(*statePtr).davePtr=davePtr
;
tvlct,(*rgbptr).r,(*rgbptr).g,(*rgbptr).b
ptr_free,rgbptr
;
end
