;$Id: read_dcs.pro,v 1.1 2002/09/17 14:57:18 jmspark Exp $
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
; READING
;
;	s = _read_dcs_one( file )
;	s = _read_dcs_files( files )
;	s = read_dcs( files )
;

;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************
function _read_dcs_one, file

	dave_makeDavePtr, davePtr = davePtr
	dcs_defineDaveStructure,davePtr=davePtr

	; Read the DCS file.
	nofirstline = 0
	omithisto = 0
	dcs_read_binaryOctavefile $
		, file, nofirstline, omithisto $		; INPUT
		, variables_read, unknowns,error $		; OUTPUT
		, davePtr=davePtr

	; Specific Pointer.
	sPtr = (*(*davePtr).dataStrPtr).specificPtr

	; Data.
	f = (*(*(*davePtr).dataStrPtr).commonStr.histPtr).qty
	angle = *(*sPtr).angle_propsPtr
	hw = *(*sPtr).energy_propsPtr
	DT = reform( (*sPtr).tchanlook[0,*] )

	; Find the temperature.
	Tidx = where( (*sPtr).temp_sample gt 0, nT )
	if nT ge 1 then temperature = total( ((*sPtr).temp_sample)[Tidx] ) / nT $
	else temperature = 0d

	; Monitor Counts, Duration, and wavelength
	monitor = long( (*sPtr).detsum[916] )
	duration = (*sPtr).duration
	wavelength = (*sPtr).ch_wl

	heap_free, davePtr

	return, $
		{ f:f $
		, angle:angle $
		, hw:hw $
		, DT:DT $

		, monitor:monitor $
		, duration:duration $
		, temperature:temperature $
		, wavelength:wavelength $
		}

end

function _read_dcs_files, files

	; Define system variables.
	defsysv, '!dave_defaults', exists=exists
	if not exists then begin
		cd, current=directory
		pdd=ptr_new({datDir:directory,workDir:directory,PDFReader:''})
		defsysv,'!dave_defaults',pdd
	endif
	davesysvars
	dcs_sysvars

	n_files = n_elements( files )
	f = 0d
	monitor = 0l
	duration = 0d
	sum_temperature = 0d
	for i = 0, n_files - 1 do begin

		s = _read_dcs_one( files[i] )
		f = f + s.f

		monitor = monitor + s.monitor
		duration = duration + s.duration
		sum_temperature = sum_temperature + s.temperature * s.monitor

	endfor
	temperature = sum_temperature / monitor

	; Sort and Select the useful part.
	sortkey = sort( s.angle )
	angle = s.angle[ sortkey ]
	f = transpose( f[0:999, sortkey ] )
	hw = s.hw[0:1000]
	DT = s.DT[0:999]

	; Change to 2D matrix.
	yv = replicate( 1d, n_elements( hw ) )
	xv = replicate( 1d, n_elements( angle ) )
	angle = angle # yv
	hw = xv # hw
	DT = xv # DT

	; Angle to Q conversion.
	Ki = 2d * !dpi / s.wavelength
	Kf2 = ( 2.0717 * Ki^2 - hw ) / 2.0717
	Kf = sqrt( Kf2 )
	q = sqrt( Ki^2 + Kf2 - 2d * Ki * Kf * cos( !dtor * angle ) ) * $
		angle / abs( angle )

	return, $
		{ f:f $
		, angle:angle $
		, q:q $
		, hw:hw $
		, DT:DT $

		, monitor:double( monitor ) $
		, duration:duration $
		, temperature:temperature $
		, wavelength:s.wavelength $
		}

end

function read_dcs, files

	if n_elements( files ) eq 0 then files = dave_dialog_pickfile(/multiple_files, /read)
	if files[0] eq '' then return, 0
	s = _read_dcs_files( files )

	return, s

end
;*****************************************************************************************
;23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
;       10        20        30        40        50        60        70        80        90
;*****************************************************************************************